#include <Dialogs.h>
#include <GestaltEqu.h>
#include <StandardFile.h>
#include "proc_mmu.h"

#define TrapMask 0x0800

static short NumToolboxTraps( void )
{
	if (GetToolTrapAddress(0xA86E) ==
			GetToolTrapAddress(0xAA6E))
		return(0x0200);
	else
		return(0x0400);
}

static TrapType GetTrapType(short theTrap)
{

	if ((theTrap & TrapMask) > 0)
		return(ToolTrap);
	else
		return(OSTrap);

}

static Boolean TrapAvailable(short theTrap)
{

	TrapType	tType;

	tType = GetTrapType(theTrap);
	if (tType == ToolTrap)
		theTrap = theTrap & 0x07FF;
	if (theTrap >= NumToolboxTraps())
		theTrap = 0xA89F;

	return ((tType == ToolTrap?GetToolTrapAddress(theTrap):GetOSTrapAddress(theTrap))!=
			GetToolTrapAddress(0xA89F));
}

long mmutype;

void probe()
	{
	Str255 name,name2;
	int major = 5,minor,cnt;
	long gestaltAnswer,physram,logram;
	if (TrapAvailable(0xA1AD))
		{
		Gestalt(gestaltSystemVersion, &gestaltAnswer);
		major = gestaltAnswer>>8;
		minor = gestaltAnswer&255;
		}
	if (major < 7)
		{
		ParamText((unsigned char *)"\p, Sorry, System 7 required",0,0,0);
		Alert(129, 0);
		ExitToShell();		
		}
	Gestalt(gestaltMMUType, &mmutype);
	if ((mmutype != gestalt68030MMU) && (mmutype != gestalt68040MMU))
		{
		ParamText("\pSorry, Only the 68030/68040","\pMMUs are supported","\pin this version",0);
		Alert(129, 0);
		ExitToShell();		
		}
	Gestalt(gestaltAddressingModeAttr, &gestaltAnswer);
	if (~gestaltAnswer & ((1<<gestalt32BitAddressing)|(1<<gestalt32BitSysZone)|(1<<gestalt32BitCapable)))
		{
		ParamText("\pIdeally, you should","\pReboot in 32-bit mode",0,0);
		Alert(129, 0);
//		ExitToShell();		
		}
	Gestalt(gestaltLogicalPageSize, &gestaltAnswer);
	if (gestaltAnswer != 8192)
		{
		ParamText("\pSupport for page sizes","\pother than 8K is untested","\pcontinue at your own risk!",0);
		Alert(129, 0);
		}
	Gestalt(gestaltLogicalRAMSize, &logram);
	Gestalt(gestaltPhysicalRAMSize, &physram);
	if (logram > physram)
		{
		ParamText("\pSorry, you need to","\pReboot with virtual memory","\pRAM Doubler, etc","\pturned off");
		Alert(129, 0);
		ExitToShell();		
		}
#if 0
	Gestalt(gestaltMachineType, &gestaltAnswer);
	GetIndString(name, kMachineNameStrID, gestaltAnswer);
	cnt = 0;
	do
		{
		GetIndString(name2, 128, ++cnt);
		if (!memcmp(name, name2, *name)) gestaltAnswer = 0;
		}
	while (*name2 && gestaltAnswer);
	if (gestaltAnswer)
		{
		ParamText("\pThis program is untested on",name,"\pdon't blame me","\pif it crashes");
		Alert(129, 0);
		}
#endif
	}

int get_page_size(void)
	{
	switch(mmutype)
		{
		case  gestalt68030MMU:
			return 1<<((get_68030_tc()>>20)&15);
			break;
		case  gestalt68040MMU:
			return get_68040_tc()&16384?8192:4096;
			break;
		default:
			return 0;
		}
	}

long trap_patch_0()
	{
	switch(mmutype)
		{
		case  gestalt68030MMU:
			return (long)trap_patch_68030_0;
			break;
		case  gestalt68040MMU:
			return (long)trap_patch_68040_0;
			break;
		default:
			return 0;
		}	
	}

long trap_patch_1()
	{
	switch(mmutype)
		{
		case  gestalt68030MMU:
			return (long)trap_patch_68030_1;
			break;
		case  gestalt68040MMU:
			return (long)trap_patch_68040_1;
			break;
		default:
			return 0;
		}	
	}

long get_user_root()
	{
	long vec[2];
	switch(mmutype)
		{
		case  gestalt68030MMU:
			get_68030_crp(vec);
			return vec[1];
			break;
		case  gestalt68040MMU:
			get_68040_crp(vec);
			return vec[1];
			break;
		default:
			return 0;
		}		
	}

void put_user_root(long addr)
	{
	long vec[2];
	switch(mmutype)
		{
		case  gestalt68030MMU:
			get_68030_crp(vec);
			vec[1] = addr;
			put_68030_crp(vec);
			break;
		case  gestalt68040MMU:
			vec[1] = addr;
			put_68040_crp(vec);
			break;
		}		
	}

void flush_caches()
	{
	switch(mmutype)
		{
		case  gestalt68030MMU:
			flush_68030_caches();
			break;
		case  gestalt68040MMU:
			flush_68040_caches();
			break;
		}	
	}

void flush_mmu_caches()
	{
	switch(mmutype)
		{
		case  gestalt68030MMU:
			flush_68030_mmu_caches();
			break;
		case  gestalt68040MMU:
			flush_68040_mmu_caches();
			break;
		}	
	}

void	*logical_to_physical(struct mmu *crp, unsigned long addr)
	{
	switch(mmutype)
		{
		case  gestalt68030MMU:
			{
			long	level2 = crp->u.u3.level2[(addr>>22)&1023]&~255;
			if ((level2 >= (long)(crp->u.u3.level3)) 
				&& (level2 < sizeof(crp->u.u3.level3)+(long)(crp->u.u3.level3)))
				{
				long	level3 = ((long *)level2)[(addr&((1<<22)-1))/page_size]&~255;
				if (level3)
					return (void *)(level3+(addr&(page_size-1)));
				}
			}
		break;
		case  gestalt68040MMU:
			{
			long	level1 = crp->u.u4.level1[addr>>25]&~255;
			if
				(
				 (level1 >= (long)(crp->u.u4.level2)
				 ) 
				&& 
				 (level1 < sizeof(crp->u.u4.level2)+(long)(crp->u.u4.level2)
				 )
				)
				{
				long	level2 = ((long *)level1)[(addr>>18)&127]&~255;
				if ((level2 >= (long)(crp->u.u4.level3)) 
					&& (level2 < sizeof(crp->u.u4.level3)+(long)(crp->u.u4.level3)))
					{
					long	level3 = ((long *)level2)[(addr&((1<<18)-1))/page_size]&~255;
					if (level3)
						return (void *)(level3+(addr&(page_size-1)));
					}
				}
			return((void *)-1);
			}
		break;
		}	
	return((void *)-1);
	}
	
int	upper_space(struct mmu *crp, long size)
	{
	long addr,level2;
	switch(mmutype)
		{
		case  gestalt68030MMU:
			{
			for (level2 = 8; level2--; )
				{
				for (addr = (1<<22)/page_size; addr--; )
					{
					if (!crp->u.u3.level3[level2][addr])
						{
						long page = page_alloc();
						if (page) crp->u.u3.level3[level2][addr] = 0x1 + page;
						else return -1;
						}
					size -= page_size;
					if (size <= 0) return 0;
					}
				}
			return -1;
			}
		case  gestalt68040MMU:
			{
			for (level2 = 128; level2--; )
				{
				for (addr = (1<<18)/page_size; addr--; )
					{
					if (!crp->u.u4.level3[level2][addr])
						{
						long page = page_alloc();
						if (page) crp->u.u4.level3[level2][addr] = 0x39 + page;
						else return -1;
						}
					size -= page_size;
					if (size <= 0) return 0;
					}
				}
			return -1;			
			}
		}
	}

int	lower_space(struct mmu *crp, long size)
	{
	long addr,level2;
	switch(mmutype)
		{
		case  gestalt68030MMU:
			{
			for (level2 = 0; level2 < 8; level2++)
				{
				for (addr = 0; addr < (1<<22)/page_size; addr++)
					{
					if (!crp->u.u3.level3[level2][addr])
						{
						long page = page_alloc();
						if (page) crp->u.u3.level3[level2][addr] = 0x1 + page;
						else return -1;
						}
					size -= page_size;
					if (size <= 0) return 0;
					}
				}
			return -1;
			}
		case  gestalt68040MMU:
			{
			for (level2 = 0; level2 < 128; level2++)
				{
				for (addr = 0; addr < (1<<18)/page_size; addr++)
					{
					if (!crp->u.u4.level3[level2][addr])
						{
						long page = page_alloc();
						if (page) crp->u.u4.level3[level2][addr] = 0x39 + page;
						else return -1;
						}
					size -= page_size;
					if (size <= 0) return 0;
					}
				}
			return -1;
			}
		default:
			return -1;
		}
	}

void free_space(struct mmu *crp)
	{
	long addr,level2;
	flush_caches();
	switch(mmutype)
		{
		case  gestalt68030MMU:
			{
			for (level2 = 0; level2 < 8; level2++)
				{
				for (addr = 0; addr < (1<<22)/page_size; addr++)
					{
					page_free(crp->u.u3.level3[level2][addr]&~255);
					}
				}
			}
		break;
		case  gestalt68040MMU:
			{
			for (level2 = 0; level2 < 128; level2++)
				{
				for (addr = 0; addr < (1<<18)/page_size; addr++)
					{
					page_free(crp->u.u4.level3[level2][addr]&~255);
					}
				}
			}
		break;
		}
	}

void wipeout(struct mmu *crp)
	{
	long addr;
	switch(mmutype)
		{
		case  gestalt68030MMU:
			{
			for (addr = 0; addr < 8; addr++)
				{
				crp->u.u3.level2[addr] = 0xA + (long)&(crp->u.u3.level3[addr][0]);
				}
			}
		break;
		case  gestalt68040MMU:
			{
			for (addr = 0; addr < 128; addr++)
				{
				crp->u.u4.level1[addr] = 0x18;
				crp->u.u4.level2[addr] = 0xA + (long)&(crp->u.u4.level3[addr][0]);
				}
			crp->u.u4.level1[0] = 0xA + (long)(crp->u.u4.level2);
			}
		break;
		}
	}
